@@ -1,12 +1,12 @@
-import imghdr
 import os.path
 import time
 from operator import itemgetter
 from typing import Any, Dict, List, Optional, Union
 
+import filetype
 import requests.exceptions
 
-from bgmi.config import MAX_PAGE, write_config
+from bgmi.config import Source, cfg
 from bgmi.lib.constants import BANGUMI_UPDATE_TIME, SUPPORT_WEBSITE
 from bgmi.lib.download import Episode, download_prepare
 from bgmi.lib.fetch import website
@@ -43,7 +43,7 @@ from bgmi.utils import (
 ControllerResult = Dict[str, Any]
 
 
-def add(name: str, episode: int = None) -> ControllerResult:
+def add(name: str, episode: Optional[int] = None) -> ControllerResult:
     """
     ret.name :str
     """
@@ -77,7 +77,7 @@ def add(name: str, episode: int = None) -> ControllerResult:
 
     Filter.get_or_create(bangumi_name=name)
 
-    max_episode, _ = website.get_maximum_episode(bangumi_obj, max_page=int(MAX_PAGE))
+    max_episode, _ = website.get_maximum_episode(bangumi_obj, max_page=cfg.max_path)
     followed_obj.episode = max_episode if episode is None else episode
 
     followed_obj.save()
@@ -108,16 +108,12 @@ def filter_(
         Followed.get(bangumi_name=bangumi_obj.name)
     except Followed.DoesNotExist:
         result["status"] = "error"
-        result[
-            "message"
-        ] = "Bangumi {name} has not subscribed, try 'bgmi add \"{name}\"'.".format(
+        result["message"] = "Bangumi {name} has not subscribed, try 'bgmi add \"{name}\"'.".format(
             name=bangumi_obj.name
         )
         return result
 
-    followed_filter_obj, is_this_obj_created = Filter.get_or_create(
-        bangumi_name=bangumi_obj.name
-    )
+    followed_filter_obj, is_this_obj_created = Filter.get_or_create(bangumi_name=bangumi_obj.name)
 
     if is_this_obj_created:
         followed_filter_obj.save()
@@ -125,13 +121,9 @@ def filter_(
     if subtitle is not None:
         _subtitle = [s.strip() for s in subtitle.split(",")]
         _subtitle = [s["id"] for s in Subtitle.get_subtitle_by_name(_subtitle)]
-        subtitle_list = [
-            s.split(".")[0] for s in bangumi_obj.subtitle_group.split(", ") if "." in s
-        ]
+        subtitle_list = [s.split(".")[0] for s in bangumi_obj.subtitle_group.split(", ") if "." in s]
         subtitle_list.extend(bangumi_obj.subtitle_group.split(", "))
-        followed_filter_obj.subtitle = ", ".join(
-            filter(lambda s: s in subtitle_list, _subtitle)
-        )
+        followed_filter_obj.subtitle = ", ".join(filter(lambda s: s in subtitle_list, _subtitle))
 
     if include is not None:
         followed_filter_obj.include = include
@@ -143,20 +135,12 @@ def filter_(
         followed_filter_obj.regex = regex
 
     followed_filter_obj.save()
-    subtitle_list = [
-        s["name"]
-        for s in Subtitle.get_subtitle_by_id(bangumi_obj.subtitle_group.split(", "))
-    ]
+    subtitle_list = [s["name"] for s in Subtitle.get_subtitle_by_id(bangumi_obj.subtitle_group.split(", "))]
 
     result["data"] = {
         "name": bangumi_obj.name,
         "subtitle_group": subtitle_list,
-        "followed": [
-            s["name"]
-            for s in Subtitle.get_subtitle_by_id(
-                followed_filter_obj.subtitle.split(", ")
-            )
-        ]
+        "followed": [s["name"] for s in Subtitle.get_subtitle_by_id(followed_filter_obj.subtitle.split(", "))]
         if followed_filter_obj.subtitle
         else [],
         "include": followed_filter_obj.include,
@@ -167,9 +151,7 @@ def filter_(
     return result
 
 
-def delete(
-    name: str = "", clear_all: bool = False, batch: bool = False
-) -> ControllerResult:
+def delete(name: str = "", clear_all: bool = False, batch: bool = False) -> ControllerResult:
     """
     :param name:
     :param clear_all:
@@ -225,7 +207,7 @@ def cal(
             for bangumi in daily_bangumi:
                 _, file_path = convert_cover_url_to_path(bangumi["cover"])
 
-                if not (os.path.exists(file_path) and bool(imghdr.what(file_path))):
+                if not (os.path.exists(file_path) and filetype.is_image(file_path)):
                     cover_to_be_download.append(bangumi["cover"])
 
         if cover_to_be_download:
@@ -245,9 +227,7 @@ def cal(
             bangumi["cover"] = normalize_path(bangumi["cover"])
             subtitle_group = [
                 {"name": x["name"], "id": x["id"]}
-                for x in Subtitle.get_subtitle_by_id(
-                    bangumi["subtitle_group"].split(", " "")
-                )
+                for x in Subtitle.get_subtitle_by_id(bangumi["subtitle_group"].split(", " ""))
             ]
 
             r[day][index]["subtitle_group"] = subtitle_group
@@ -256,9 +236,7 @@ def cal(
 
 
 def download(name: str, title: str, episode: int, download_url: str) -> None:
-    download_prepare(
-        [Episode(name=name, title=title, episode=episode, download=download_url)]
-    )
+    download_prepare([Episode(name=name, title=title, episode=episode, download=download_url)])
 
 
 def mark(name: str, episode: int) -> ControllerResult:
@@ -291,18 +269,23 @@ def mark(name: str, episode: int) -> ControllerResult:
 
 def search(
     keyword: str,
-    count: Union[str, int] = MAX_PAGE,
-    regex: str = None,
+    count: Union[str, int] = cfg.max_path,
+    regex: Optional[str] = None,
     dupe: bool = False,
-    min_episode: int = None,
-    max_episode: int = None,
+    min_episode: Optional[int] = None,
+    max_episode: Optional[int] = None,
+    tag: bool = False,
+    subtitle: Optional[str] = None,
 ) -> ControllerResult:
     try:
         count = int(count)
     except (TypeError, ValueError):
         count = 3
     try:
-        data = website.search_by_keyword(keyword, count=count)
+        if tag:
+            data = website.search_by_tag(keyword, subtitle=subtitle, count=count)
+        else:
+            data = website.search_by_keyword(keyword, count=count)
         data = episode_filter_regex(data, regex=regex)
         if min_episode is not None:
             data = [x for x in data if x.episode >= min_episode]
@@ -347,43 +330,20 @@ def source(data_source: str) -> ControllerResult:
     result = {}
     if data_source in list(map(itemgetter("id"), SUPPORT_WEBSITE)):
         recreate_source_relatively_table()
-        write_config("DATA_SOURCE", data_source)
+        cfg.data_source = Source(data_source)
+        cfg.save()
         print_success("data source switch succeeds")
         result["status"] = "success"
-        result[
-            "message"
-        ] = f"you have successfully change your data source to {data_source}"
+        result["message"] = f"you have successfully change your data source to {data_source}"
     else:
         result["status"] = "error"
-        result[
-            "message"
-        ] = "please check your input. data source should be one of {}".format(
+        result["message"] = "please check your input. data source should be one of {}".format(
             [x["id"] for x in SUPPORT_WEBSITE]
         )
     return result
 
 
-def config(name: Optional[str] = None, value: Optional[str] = None) -> ControllerResult:
-    if name == "DATA_SOURCE":
-        error_message = (
-            "you can't change data source in this way. "
-            "please use `bgmi source ${data source}` in cli"
-        )
-        result = {
-            "status": "error",
-            "message": error_message,
-            "data": write_config()["data"],
-        }
-        return result
-    r = write_config(name, value)
-    if name == "ADMIN_TOKEN":
-        r["message"] = "you need to restart your bgmi_http to make new token work"
-    return r
-
-
-def update(
-    name: List[str], download: Any = None, not_ignore: bool = False
-) -> ControllerResult:
+def update(name: List[str], download: Optional[Any] = None, not_ignore: bool = False) -> ControllerResult:
     logger.debug("updating bangumi info with args: download: %r", download)
     result: Dict[str, Any] = {
         "status": "info",
@@ -436,23 +396,17 @@ def update(
         try:
             bangumi_obj = Bangumi.get(name=subscribe["bangumi_name"])
         except Bangumi.DoesNotExist:
-            print_error(
-                "Bangumi<{}> does not exists.".format(subscribe["bangumi_name"]),
-                exit_=False,
-            )
+            logger.error("Bangumi<{}> does not exists.", subscribe["bangumi_name"])
             continue
         try:
             followed_obj = Followed.get(bangumi_name=subscribe["bangumi_name"])
         except Followed.DoesNotExist:
-            print_error(
-                "Bangumi<{}> is not followed.".format(subscribe["bangumi_name"]),
-                exit_=False,
-            )
+            logger.error("Bangumi<{}> is not followed.", subscribe["bangumi_name"])
             continue
 
         try:
             episode, all_episode_data = website.get_maximum_episode(
-                bangumi=bangumi_obj, ignore_old_row=ignore, max_page=int(MAX_PAGE)
+                bangumi=bangumi_obj, ignore_old_row=ignore, max_page=cfg.max_path
             )
         except requests.exceptions.ConnectionError as e:
             print_warning(f"error {e} to fetch {bangumi_obj.name}, skip")
@@ -463,16 +417,12 @@ def update(
                 episode_range = download
             else:
                 episode_range = range(subscribe["episode"] + 1, episode + 1)
-                print_success(
-                    f"{subscribe['bangumi_name']} updated, episode: {episode:d}"
-                )
+                print_success(f"{subscribe['bangumi_name']} updated, episode: {episode:d}")
                 followed_obj.episode = episode
                 followed_obj.status = STATUS_UPDATED
                 followed_obj.updated_time = int(time.time())
                 followed_obj.save()
-                result["data"]["updated"].append(
-                    {"bangumi": subscribe["bangumi_name"], "episode": episode}
-                )
+                result["data"]["updated"].append({"bangumi": subscribe["bangumi_name"], "episode": episode})
 
             for i in episode_range:
                 for epi in all_episode_data:
@@ -487,13 +437,7 @@ def update(
         print_info("Re-downloading ...")
         download_prepare(
             [
-                Episode(
-                    **{
-                        key: value
-                        for key, value in x.items()
-                        if key not in ["id", "status"]
-                    }
-                )
+                Episode(**{key: value for key, value in x.items() if key not in ["id", "status"]})
                 for x in Download.get_all_downloads(status=STATUS_NOT_DOWNLOAD)
             ]
         )
@@ -545,16 +489,11 @@ def list_() -> ControllerResult:
         if followed_bangumi[weekday.lower()]:
             result["message"] += f"{GREEN}{weekday}. {COLOR_END}"
             for j, bangumi in enumerate(followed_bangumi[weekday.lower()]):
-                if (
-                    bangumi["status"] in (STATUS_UPDATED, STATUS_FOLLOWED)
-                    and "episode" in bangumi
-                ):
+                if bangumi["status"] in (STATUS_UPDATED, STATUS_FOLLOWED) and "episode" in bangumi:
                     bangumi["name"] = f"{bangumi['name']}({bangumi['episode']:d})"
                 if j > 0:
                     result["message"] += " " * 5
                 f = [x["name"] for x in bangumi["subtitle_group"]]
-                result["message"] += "{}: {}\n".format(
-                    bangumi["name"], ", ".join(f) if f else "<None>"
-                )
+                result["message"] += "{}: {}\n".format(bangumi["name"], ", ".join(f) if f else "<None>")
 
     return result
